@@ -76,6 +76,8 @@
 #include <mail_addr.h>
 #include <canon_addr.h>
 #include <mail_addr_find.h>
+#include <mail_proto.h>
+#include <dsn_mask.h>
 
 /* Application-specific. */
 
@@ -91,7 +93,14 @@ void    cleanup_addr_sender(CLEANUP_STATE *state, const char *buf)
     VSTRING *clean_addr = vstring_alloc(100);
     const char *bcc;
 
-    cleanup_rewrite_internal(clean_addr, buf);
+    /*
+     * Note: an unqualified envelope address is for all practical purposes
+     * equivalent to a fully qualified local address, both for delivery and
+     * for replying. Having to support both forms is error prone, therefore
+     * an incomplete envelope address is rewritten to fully qualified form in
+     * the local domain context.
+     */
+    cleanup_rewrite_internal(MAIL_ATTR_RWR_LOCAL, clean_addr, buf);
     if (strncasecmp(STR(clean_addr), MAIL_ADDR_MAIL_DAEMON "@",
 		    sizeof(MAIL_ADDR_MAIL_DAEMON)) == 0) {
 	canon_addr_internal(state->temp1, MAIL_ADDR_MAIL_DAEMON);
@@ -130,7 +139,15 @@ void    cleanup_addr_recipient(CLEANUP_STATE *state, const char *buf)
     VSTRING *clean_addr = vstring_alloc(100);
     const char *bcc;
 
-    cleanup_rewrite_internal(clean_addr, *buf ? buf : var_empty_addr);
+    /*
+     * Note: an unqualified envelope address is for all practical purposes
+     * equivalent to a fully qualified local address, both for delivery and
+     * for replying. Having to support both forms is error prone, therefore
+     * an incomplete envelope address is rewritten to fully qualified form in
+     * the local domain context.
+     */
+    cleanup_rewrite_internal(MAIL_ATTR_RWR_LOCAL,
+			     clean_addr, *buf ? buf : var_empty_addr);
     if (state->flags & CLEANUP_FLAG_MAP_OK) {
 	if (cleanup_rcpt_canon_maps
 	    && (cleanup_rcpt_canon_flags & CLEANUP_CANON_FLAG_ENV_RCPT))
@@ -144,7 +161,8 @@ void    cleanup_addr_recipient(CLEANUP_STATE *state, const char *buf)
 	    && (cleanup_masq_flags & CLEANUP_MASQ_FLAG_ENV_RCPT))
 	    cleanup_masquerade_internal(clean_addr, cleanup_masq_domains);
     }
-    cleanup_out_recipient(state, state->orig_rcpt, STR(clean_addr));
+    cleanup_out_recipient(state, state->dsn_orcpt, state->dsn_notify,
+			  state->orig_rcpt, STR(clean_addr));
     if (state->recip == 0)
 	state->recip = mystrdup(STR(clean_addr));
     if ((state->flags & CLEANUP_FLAG_BCC_OK)
@@ -162,7 +180,11 @@ void    cleanup_addr_bcc(CLEANUP_STATE *state, const char *bcc)
 {
     VSTRING *clean_addr = vstring_alloc(100);
 
-    cleanup_rewrite_internal(clean_addr, bcc);
+    /*
+     * Note: BCC addresses are supplied locally, and must be rewritten in the
+     * local address rewriting context.
+     */
+    cleanup_rewrite_internal(MAIL_ATTR_RWR_LOCAL, clean_addr, bcc);
     if (state->flags & CLEANUP_FLAG_MAP_OK) {
 	if (cleanup_rcpt_canon_maps
 	    && (cleanup_rcpt_canon_flags & CLEANUP_CANON_FLAG_ENV_RCPT))
@@ -176,6 +198,7 @@ void    cleanup_addr_bcc(CLEANUP_STATE *state, const char *bcc)
 	    && (cleanup_masq_flags & CLEANUP_MASQ_FLAG_ENV_RCPT))
 	    cleanup_masquerade_internal(clean_addr, cleanup_masq_domains);
     }
-    cleanup_out_recipient(state, STR(clean_addr), STR(clean_addr));
+    cleanup_out_recipient(state, (char *) 0, DSN_NOTIFY_NEVER,
+			  STR(clean_addr), STR(clean_addr));
     vstring_free(clean_addr);
 }
