@@ -46,10 +46,6 @@
 #include <stdlib.h>
 #include <ctype.h>
 
-#ifdef STRCASECMP_IN_STRINGS_H
-#include <strings.h>
-#endif
-
 /* Utility library. */
 
 #include <msg.h>
@@ -118,12 +114,17 @@ static void cleanup_envelope_process(CLEANUP_STATE *state, int type,
     int     mapped_type = type;
     const char *mapped_buf = buf;
 
+#ifdef DELAY_ACTION
+    int     defer_delay;
+
+#endif
+
     if (msg_verbose)
 	msg_info("initial envelope %c %.*s", type, (int) len, buf);
 
     if (type == REC_TYPE_FLGS) {
 	/* Not part of queue file format. */
-	extra_opts = atol(buf);
+	extra_opts = atoi(buf);
 	if (extra_opts & ~CLEANUP_FLAG_MASK_EXTRA)
 	    msg_warn("%s: ignoring bad extra flags: 0x%x",
 		     state->queue_id, extra_opts);
@@ -132,6 +133,18 @@ static void cleanup_envelope_process(CLEANUP_STATE *state, int type,
 	return;
     }
 
+#ifdef DELAY_ACTION
+    if (type == REC_TYPE_DELAY) {
+	/* Not part of queue file format. */
+	defer_delay = atoi(buf);
+	if (defer_delay <= 0)
+	    msg_warn("%s: ignoring bad delay time: %s", state->queue_id, buf);
+	else
+	    state->defer_delay = defer_delay;
+	return;
+    }
+#endif
+
     /*
      * Map DSN attribute name to pseudo record type so that we don't have to
      * pollute the queue file with records that are incompatible with past
