@@ -41,6 +41,10 @@
 #include <string.h>
 #include <ctype.h>
 
+#ifdef STRCASECMP_IN_STRINGS_H
+#include <strings.h>
+#endif
+
 /* Utility library. */
 
 #include "mymalloc.h"
