@@ -9,7 +9,7 @@
 /*	VSTREAM	*safe_open(path, flags, mode, st, user, group, why)
 /*	const char *path;
 /*	int	flags;
-/*	int	mode;
+/*	mode_t	mode;
 /*	struct stat *st;
 /*	uid_t	user;
 /*	gid_t	group;
@@ -174,8 +174,8 @@ static VSTREAM *safe_open_exist(const char *path, int flags,
 
 /* safe_open_create - create new file */
 
-static VSTREAM *safe_open_create(const char *path, int flags, int mode,
-	            struct stat * st, uid_t user, gid_t group, VSTRING *why)
+static VSTREAM *safe_open_create(const char *path, int flags, mode_t mode,
+	            struct stat * st, uid_t user, uid_t group, VSTRING *why)
 {
     VSTREAM *fp;
 
@@ -188,12 +188,6 @@ static VSTREAM *safe_open_create(const char *path, int flags, int mode,
 	return (0);
     }
 
-    /*
-     * Optionally look up the file attributes.
-     */
-    if (st != 0 && fstat(vstream_fileno(fp), st) < 0)
-	msg_fatal("%s: bad open file status: %m", path);
-
     /*
      * Optionally change ownership after creating a new file. If there is a
      * problem we should not attempt to delete the file. Something else may
@@ -206,6 +200,12 @@ static VSTREAM *safe_open_create(const char *path, int flags, int mode,
 	msg_warn("%s: cannot change file ownership: %m", path);
     }
 
+    /*
+     * Optionally look up the file attributes.
+     */
+    if (st != 0 && fstat(vstream_fileno(fp), st) < 0)
+	msg_fatal("%s: bad open file status: %m", path);
+
     /*
      * We are almost there...
      */
@@ -222,7 +222,7 @@ static VSTREAM *safe_open_create(const char *path, int flags, int mode,
 
 /* safe_open - safely open or create file */
 
-VSTREAM *safe_open(const char *path, int flags, int mode,
+VSTREAM *safe_open(const char *path, int flags, mode_t mode,
 	            struct stat * st, uid_t user, gid_t group, VSTRING *why)
 {
     VSTREAM *fp;
