@@ -9,11 +9,18 @@
 /*	VSTRING	*unescape(result, input)
 /*	VSTRING	*result;
 /*	const char *input;
+/*
+/*	VSTRING	*escape(result, input, len)
+/*	VSTRING	*result;
+/*	const char *input;
+/*	ssize_t len;
 /* DESCRIPTION
 /*	unescape() translates C-like escape sequences in the null-terminated
 /*	string \fIinput\fR and places the result in \fIresult\fR. The result
 /*	is null-terminated, and is the function result value.
 /*
+/*	escape() does the reverse transformation.
+/*
 /*	Escape sequences and their translations:
 /* .IP \ea
 /*	Bell character.
@@ -121,6 +128,50 @@ VSTRING *unescape(VSTRING *result, const char *data)
     return (result);
 }
 
+/* escape - reverse transformation */
+
+VSTRING *escape(VSTRING *result, const char *data, ssize_t len)
+{
+    int     ch;
+
+    VSTRING_RESET(result);
+    while (len-- > 0) {
+	ch = *UCHAR(data++);
+	if (ISASCII(ch)) {
+	    if (ISPRINT(ch)) {
+		if (ch == '\\')
+		    VSTRING_ADDCH(result, ch);
+		VSTRING_ADDCH(result, ch);
+		continue;
+	    } else if (ch == '\a') {		/* \a -> audible bell */
+		vstring_strcat(result, "\a");
+		continue;
+	    } else if (ch == '\b') {		/* \b -> backspace */
+		vstring_strcat(result, "\b");
+		continue;
+	    } else if (ch == '\f') {		/* \f -> formfeed */
+		vstring_strcat(result, "\f");
+		continue;
+	    } else if (ch == '\n') {		/* \n -> newline */
+		vstring_strcat(result, "\n");
+		continue;
+	    } else if (ch == '\r') {		/* \r -> carriagereturn */
+		vstring_strcat(result, "\r");
+		continue;
+	    } else if (ch == '\t') {		/* \t -> horizontal tab */
+		vstring_strcat(result, "\t");
+		continue;
+	    } else if (ch == '\v') {		/* \v -> vertical tab */
+		vstring_strcat(result, "\v");
+		continue;
+	    }
+	}
+	vstring_sprintf_append(result, "\\%03d", ch);
+    }
+    VSTRING_TERMINATE(result);
+    return (result);
+}
+
 #ifdef TEST
 
 #include <stdlib.h>
